@@ -16,8 +16,6 @@
 #include "nm-core-internal.h"
 #include "nm-dbus-helpers.h"
 
-#include "introspection/org.freedesktop.NetworkManager.Device.WifiP2P.h"
-
 /*****************************************************************************/
 
 NM_GOBJECT_PROPERTIES_DEFINE_BASE (
@@ -35,11 +33,8 @@ enum {
 static guint signals[LAST_SIGNAL] = { 0 };
 
 typedef struct {
-	NMDBusDeviceWifiP2P *proxy;
-
+	NMLDBusPropertyAO peers;
 	char *hw_address;
-
-	GPtrArray   *peers;
 } NMDeviceWifiP2PPrivate;
 
 struct _NMDeviceWifiP2P {
@@ -93,7 +88,7 @@ nm_device_wifi_p2p_get_peers (NMDeviceWifiP2P *device)
 {
 	g_return_val_if_fail (NM_IS_DEVICE_WIFI_P2P (device), NULL);
 
-	return NM_DEVICE_WIFI_P2P_GET_PRIVATE (device)->peers;
+	return nml_dbus_property_ao_get_objs_as_ptrarray (&NM_DEVICE_WIFI_P2P_GET_PRIVATE (device)->peers);
 }
 
 /**
@@ -164,7 +159,8 @@ nm_device_wifi_p2p_start_find (NMDeviceWifiP2P *device,
 	if (!options)
 		options = g_variant_new_array (G_VARIANT_TYPE ("{sv}"), NULL, 0);
 
-	_nm_object_dbus_call (device,
+	_nm_client_dbus_call (_nm_object_get_client (device),
+	                      device,
 	                      nm_device_wifi_p2p_start_find,
 	                      cancellable,
 	                      callback,
@@ -222,7 +218,8 @@ nm_device_wifi_p2p_stop_find (NMDeviceWifiP2P     *device,
 	g_return_if_fail (NM_IS_DEVICE_WIFI_P2P (device));
 	g_return_if_fail (!cancellable || G_IS_CANCELLABLE (cancellable));
 
-	_nm_object_dbus_call (device,
+	_nm_client_dbus_call (_nm_object_get_client (device),
+	                      device,
 	                      nm_device_wifi_p2p_stop_find,
 	                      cancellable,
 	                      callback,
@@ -260,21 +257,6 @@ nm_device_wifi_p2p_stop_find_finish (NMDeviceWifiP2P  *device,
 	return g_task_propagate_boolean (G_TASK (result), error);
 }
 
-static void
-clean_up_peers (NMDeviceWifiP2P *self)
-{
-	NMDeviceWifiP2PPrivate *priv = NM_DEVICE_WIFI_P2P_GET_PRIVATE (self);
-
-	while (priv->peers->len > 0) {
-		NMWifiP2PPeer *peer;
-
-		peer = priv->peers->pdata[priv->peers->len - 1];
-		g_ptr_array_remove_index (priv->peers, priv->peers->len - 1);
-
-		g_signal_emit (self, signals[PEER_REMOVED], 0, peer);
-	}
-}
-
 static gboolean
 connection_compatible (NMDevice *device, NMConnection *connection, GError **error)
 {
@@ -310,6 +292,24 @@ get_type_description (NMDevice *device)
 
 /*****************************************************************************/
 
+static void
+_property_ao_notify_changed_peers_cb (NMLDBusPropertyAO *pr_ao,
+                                      NMClient *client,
+                                      NMObject *nmobj,
+                                      gboolean is_added /* or else removed */)
+{
+	_nm_client_notify_event_queue_emit_obj_signal (client,
+	                                               G_OBJECT (pr_ao->owner_dbobj->nmobj),
+	                                               nmobj,
+	                                               is_added,
+	                                               10,
+	                                                 is_added
+	                                               ? signals[PEER_ADDED]
+	                                               : signals[PEER_REMOVED]);
+}
+
+/*****************************************************************************/
+
 static void
 get_property (GObject *object,
               guint prop_id,
@@ -331,38 +331,11 @@ get_property (GObject *object,
 	}
 }
 
+/*****************************************************************************/
+
 static void
 nm_device_wifi_p2p_init (NMDeviceWifiP2P *device)
 {
-	NMDeviceWifiP2PPrivate *priv = NM_DEVICE_WIFI_P2P_GET_PRIVATE (device);
-
-	priv->peers = g_ptr_array_new ();
-}
-
-static void
-init_dbus (NMObject *object)
-{
-	NMDeviceWifiP2PPrivate *priv = NM_DEVICE_WIFI_P2P_GET_PRIVATE (object);
-	const NMPropertiesInfo property_info[] = {
-		{ NM_DEVICE_WIFI_P2P_HW_ADDRESS,           &priv->hw_address },
-		{ NM_DEVICE_WIFI_P2P_PEERS,                &priv->peers, NULL, NM_TYPE_WIFI_P2P_PEER, "peer" },
-		{ NULL },
-	};
-
-	NM_OBJECT_CLASS (nm_device_wifi_p2p_parent_class)->init_dbus (object);
-
-	priv->proxy = NMDBUS_DEVICE_WIFI_P2P (_nm_object_get_proxy (object, NM_DBUS_INTERFACE_DEVICE_WIFI_P2P));
-	_nm_object_register_properties (object,
-	                                NM_DBUS_INTERFACE_DEVICE_WIFI_P2P,
-	                                property_info);
-}
-
-static void
-dispose (GObject *object)
-{
-	clean_up_peers (NM_DEVICE_WIFI_P2P (object));
-
-	G_OBJECT_CLASS (nm_device_wifi_p2p_parent_class)->dispose (object);
 }
 
 static void
@@ -370,32 +343,40 @@ finalize (GObject *object)
 {
 	NMDeviceWifiP2PPrivate *priv = NM_DEVICE_WIFI_P2P_GET_PRIVATE (object);
 
-	g_clear_object (&priv->proxy);
 	g_free (priv->hw_address);
-	if (priv->peers)
-		g_ptr_array_unref (priv->peers);
 
 	G_OBJECT_CLASS (nm_device_wifi_p2p_parent_class)->finalize (object);
 }
 
+const NMLDBusMetaIface _nml_dbus_meta_iface_nm_device_wifip2p = NML_DBUS_META_IFACE_INIT_PROP (
+	NM_DBUS_INTERFACE_DEVICE_WIFI_P2P,
+	nm_device_wifi_p2p_get_type,
+	NML_DBUS_META_INTERFACE_PRIO_INSTANTIATE_HIGH,
+	NML_DBUS_META_IFACE_DBUS_PROPERTIES (
+		NML_DBUS_META_PROPERTY_INIT_S       ("HwAddress", PROP_HW_ADDRESS, NMDeviceWifiP2P, _priv.hw_address                                                                                      ),
+		NML_DBUS_META_PROPERTY_INIT_AO_PROP ("Peers",     PROP_PEERS,      NMDeviceWifiP2P, _priv.peers,     nm_wifi_p2p_peer_get_type, .notify_changed_ao = _property_ao_notify_changed_peers_cb ),
+	),
+);
+
 static void
-nm_device_wifi_p2p_class_init (NMDeviceWifiP2PClass *wifi_class)
+nm_device_wifi_p2p_class_init (NMDeviceWifiP2PClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (wifi_class);
-	NMObjectClass *nm_object_class = NM_OBJECT_CLASS (wifi_class);
-	NMDeviceClass *device_class = NM_DEVICE_CLASS (wifi_class);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	NMObjectClass *nm_object_class = NM_OBJECT_CLASS (klass);
+	NMDeviceClass *device_class = NM_DEVICE_CLASS (klass);
 
 	object_class->get_property = get_property;
-	object_class->dispose      = dispose;
 	object_class->finalize     = finalize;
 
+	_NM_OBJECT_CLASS_INIT_PRIV_PTR_DIRECT (nm_object_class, NMDeviceWifiP2P);
+
+	_NM_OBJECT_CLASS_INIT_PROPERTY_AO_FIELDS_1 (nm_object_class, NMDeviceWifiP2PPrivate, peers);
+
 	device_class->connection_compatible = connection_compatible;
 	device_class->get_setting_type      = get_setting_type;
 	device_class->get_hw_address        = get_hw_address;
 	device_class->get_type_description  = get_type_description;
 
-	nm_object_class->init_dbus = init_dbus;
-
 	/**
 	 * NMDeviceWifiP2P:hw-address:
 	 *
@@ -422,7 +403,7 @@ nm_device_wifi_p2p_class_init (NMDeviceWifiP2PClass *wifi_class)
 	                        G_PARAM_READABLE |
 	                        G_PARAM_STATIC_STRINGS);
 
-	g_object_class_install_properties (object_class, _PROPERTY_ENUMS_LAST, obj_properties);
+	_nml_dbus_meta_class_init_with_properties (object_class, &_nml_dbus_meta_iface_nm_device_wifip2p);
 
 	/**
 	 * NMDeviceWifiP2P::peer-added:
