@@ -1,7 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1+
-/* NetworkManager -- Network link manager
- *
- * (C) Copyright 2019 Red Hat, Inc.
+/*
+ * Copyright (C) 2019 Red Hat, Inc.
  */
 
 #ifndef __NM_DBUS_AUX_H__
@@ -175,4 +174,35 @@ void nm_dbus_connection_call_get_managed_objects (GDBusConnection *dbus_connecti
 
 /*****************************************************************************/
 
+void nm_dbus_connection_call_finish_void_cb (GObject *source,
+                                             GAsyncResult *result,
+                                             gpointer user_data);
+
+void nm_dbus_connection_call_finish_void_strip_dbus_error_cb (GObject *source,
+                                                              GAsyncResult *result,
+                                                              gpointer user_data);
+
+void nm_dbus_connection_call_finish_variant_cb (GObject *source,
+                                                GAsyncResult *result,
+                                                gpointer user_data);
+
+void nm_dbus_connection_call_finish_variant_strip_dbus_error_cb (GObject *source,
+                                                                 GAsyncResult *result,
+                                                                 gpointer user_data);
+
+/*****************************************************************************/
+
+gboolean _nm_dbus_error_is (GError *error, ...) G_GNUC_NULL_TERMINATED;
+
+#define nm_dbus_error_is(error, ...) \
+	({ \
+		GError *const _error = (error); \
+		\
+		_error && _nm_dbus_error_is (_error, __VA_ARGS__, NULL); \
+	})
+
+#define NM_DBUS_ERROR_NAME_UNKNOWN_METHOD "org.freedesktop.DBus.Error.UnknownMethod"
+
+/*****************************************************************************/
+
 #endif /* __NM_DBUS_AUX_H__ */
